/*
 * (c)COPYRIGHT
 * ALL RIGHT RESERVED
 *
 * FileName : w5500.c
 * -----------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include "W5500\SPI1.h"
#include "W5500\w5500.h"
#include "W5500\w5500_if.h"
#include "W5500\w5500_delay.h"


static uint16 SSIZE; /**< Max Tx buffer size by each channel */
static uint16 RSIZE; /**< Max Rx buffer size by each channel */
void send_data_processing(SOCKET s, uint8 *wizdata, uint16 len);
void recv_data_processing(SOCKET s, uint8 *wizdata, uint16 len);
uint16 getIINCHIP_RxMAX(void);
uint16 getIINCHIP_TxMAX(void);

void IINCHIP_WRITE( uint32 addrbsb,  uint8 data);
uint8 IINCHIP_READ(uint32 addrbsb);
uint16 wiz_write_buf(uint32 addrbsb,uint8* buf,uint16 len);
uint16 wiz_read_buf(uint32 addrbsb, uint8* buf,uint16 len);


/*******************************************************************************
 * Function Name  : IINCHIP_WRITE
 * Description    : Single SPI Write functions
 * Input          : addr+bsb, data
 * Output         : None
 * Return         : None
 *******************************************************************************/
void IINCHIP_WRITE( uint32 addrbsb,  uint8 data)
{
    IINCHIP_ISR_DISABLE();                           // Interrupt Service Routine Disable
    IINCHIP_CSoff();                                 // CS=0, SPI start
    IINCHIP_SpiSendData( (addrbsb & 0x00FF0000)>>16);// Address byte 1
    IINCHIP_SpiSendData( (addrbsb & 0x0000FF00)>> 8);// Address byte 2
    IINCHIP_SpiSendData( (addrbsb & 0x000000F8) + 4);// Data write command and Write data length 1
    IINCHIP_SpiSendData(data);                       // Data write (write 1byte data)
    IINCHIP_CSon();                                  // CS=1,  SPI end
    IINCHIP_ISR_ENABLE();                            // Interrupt Service Routine Enable
}

/*******************************************************************************
 * Function Name  : IINCHIP_READ
 * Description    : Single SPI READ functions
 * Input          : addr+bsb
 * Output         : None
 * Return         : read data
 *******************************************************************************/
uint8 IINCHIP_READ(uint32 addrbsb)
{
    uint8 data = 0;
    IINCHIP_ISR_DISABLE();                           // Interrupt Service Routine Disable
    IINCHIP_CSoff();                                 // CS=0, SPI start
    IINCHIP_SpiSendData( (addrbsb & 0x00FF0000)>>16);// Address byte 1
    IINCHIP_SpiSendData( (addrbsb & 0x0000FF00)>> 8);// Address byte 2
    IINCHIP_SpiSendData( (addrbsb & 0x000000F8))    ;// Data read command and Read data length 1
    data = IINCHIP_SpiSendData(0x00);                // Data read (read 1byte data)
    IINCHIP_CSon();                                  // CS=1,  SPI end
    IINCHIP_ISR_ENABLE();                            // Interrupt Service Routine Enable
    return data;    
}

/*******************************************************************************
 * Function Name  : wiz_write_buf
 * Description    : Burst SPI Write functions
 * Input          : addr+bsb, buffer, length
 * Output         : None
 * Return         : None
 *******************************************************************************/
uint16 wiz_write_buf(uint32 addrbsb,uint8* buf,uint16 len)
{
    uint16 idx = 0;
    IINCHIP_ISR_DISABLE();
    IINCHIP_CSoff();                                 // CS=0, SPI start
    IINCHIP_SpiSendData( (addrbsb & 0x00FF0000)>>16);// Address byte 1
    IINCHIP_SpiSendData( (addrbsb & 0x0000FF00)>> 8);// Address byte 2
    IINCHIP_SpiSendData( (addrbsb & 0x000000F8) + 4);// Data write command and Write data length 1
    for(idx = 0; idx < len; idx++)                   // Write data in loop
    {
        IINCHIP_SpiSendData(buf[idx]);
    }
    IINCHIP_CSon();                                  // CS=1, SPI end
    IINCHIP_ISR_ENABLE();                            // Interrupt Service Routine Enable    

    return len;  
}

/*******************************************************************************
 * Function Name  : wiz_read_buf
 * Description    : Burst SPI read functions
 * Input          : addr+bsb, buffer, length
 * Output         : None
 * Return         : length
 *******************************************************************************/
uint16 wiz_read_buf(uint32 addrbsb, uint8* buf,uint16 len)
{
    uint16 idx = 0;
    IINCHIP_ISR_DISABLE();
    IINCHIP_CSoff();                                 // CS=0, SPI start
    IINCHIP_SpiSendData( (addrbsb & 0x00FF0000)>>16);// Address byte 1
    IINCHIP_SpiSendData( (addrbsb & 0x0000FF00)>> 8);// Address byte 2
    IINCHIP_SpiSendData( (addrbsb & 0x000000F8));    // Data write command and Write data length 1
    for(idx = 0; idx < len; idx++)                   // Write data in loop
    {
        buf[idx] = IINCHIP_SpiSendData(0x00);
    }
    IINCHIP_CSon();                                  // CS=1, SPI end
    IINCHIP_ISR_ENABLE();                            // Interrupt Service Routine Enable

    return len;
}

/*******************************************************************************
 * Function Name  : sysinit
 * Description    : set MAC Address / TX/RX BUFFER / MODE
 * Input          : MAC ADDRESS
 * Output         : None
 * Return         : None
 *******************************************************************************/
void sysinit(uint8 * mac_addr)
{

    //-- 1. Set MAC Address at Source Hardware address register
    wiz_write_buf(SHAR0, mac_addr, 6);

    //-- 2. Set TX/RX buffer size at socket transmit/receive buffer size register
    //-- ...2 -> 2048KB / 4-> 4096 / 8-> 8192...
    IINCHIP_WRITE( (Sn_TXMEM_SIZE(0)), 8); // set 8192 bytes for tx buffer
    IINCHIP_WRITE( (Sn_RXMEM_SIZE(0)), 8); // set 8192 bytes for rx buffer
    SSIZE = (int16)(8192);
    RSIZE = (int16)(8192);

    // 3. Set mode at Mode register (ping reply block, MACRAW nosize check)
    IINCHIP_WRITE(MR, MR_PB | MR_RAW_NOSIZECHK);

}
uint16 getIINCHIP_RxMAX(void)
{
    return RSIZE;
}
uint16 getIINCHIP_TxMAX(void)
{
    return SSIZE;
}


/*******************************************************************************
 * Function Name  : getSn_RX_RSR
 * Description    : This gives size of received data in receive buffer.
 * Input          : MAC ADDRESS
 * Output         : None
 * Return         : data length
 *******************************************************************************/
uint16 getSn_RX_RSR(void)
{
    uint8 s = 0;
    uint16 val=0,val1=0;
    do
    {
        val1 = IINCHIP_READ(Sn_RX_RSR0(s));
        val1 = (val1 << 8) + IINCHIP_READ(Sn_RX_RSR1(s));
        if(val1 != 0)
        {
            val = IINCHIP_READ(Sn_RX_RSR0(s));
            val = (val << 8) + IINCHIP_READ(Sn_RX_RSR1(s));
        }
    } while (val != val1);
    return val;
}

/**
  @brief   This function is being called by send() and sendto() function also.

  This function read the Tx write pointer register and after copy the data in buffer update the Tx write pointer
  register. User should read upper byte first and lower byte later to get proper value.
  */
#if 0
void send_data_processing(SOCKET s, uint8 *data, uint16 len)
{
    uint16 ptr = 0;
    uint32 addrbsb = 0;
    ptr = IINCHIP_READ( Sn_TX_WR0(s) );
    ptr = ((ptr & 0x00ff) << 8) + IINCHIP_READ(Sn_TX_WR1(s));

    addrbsb = (uint32)(ptr<<8) + (s<<5) + 0x10;
    wiz_write_buf(addrbsb, data, len);

    ptr += len;
    IINCHIP_WRITE( Sn_TX_WR0(s) ,(uint8)((ptr & 0xff00) >> 8));
    IINCHIP_WRITE( Sn_TX_WR1(s),(uint8)(ptr & 0x00ff));
}
#endif
/**
  @brief  This function is being called by recv() also.

  This function read the Rx read pointer register
  and after copy the data from receive buffer update the Rx write pointer register.
  User should read upper byte first and lower byte later to get proper value.
  */
#if 0
void recv_data_processing(SOCKET s, uint8 *data, uint16 len)
{
    uint16 ptr = 0;
    uint32 addrbsb = 0;

    ptr = IINCHIP_READ( Sn_RX_RD0(s) );
    ptr = ((ptr & 0x00ff) << 8) + IINCHIP_READ( Sn_RX_RD1(s) );

    addrbsb = (uint32)(ptr<<8) + (s<<5) + 0x18;
    wiz_read_buf(addrbsb, data, len);
    ptr += len;

    IINCHIP_WRITE( Sn_RX_RD0(s), (uint8)((ptr & 0xff00) >> 8));
    IINCHIP_WRITE( Sn_RX_RD1(s), (uint8)(ptr & 0x00ff));
}
#endif

/*******************************************************************************
 * Function Name  : macraw_open
 * Description    :  This macraw_open function open socket.
 * Input          : Node
 * Output         : None
 * Return         : None
 *******************************************************************************/
void macraw_open(void)
{
    uint8 sock_num;
    sock_num = 0;


    IINCHIP_WRITE( Sn_CR(sock_num) ,Sn_CR_CLOSE);
    while( IINCHIP_READ(Sn_CR(sock_num) ) );  
    Delay_ms(10);
    IINCHIP_WRITE(Sn_MR(sock_num) ,Sn_MR_MACRAW| // MACRAW mode
                                   Sn_MR_MIP6N|  // IPv6 packet blocking in MACRAW mode
                                   Sn_MR_MFEN);  // MAC filter enable in MACRAW mode
    IINCHIP_WRITE( Sn_CR(sock_num) ,Sn_CR_OPEN);
    while( IINCHIP_READ(Sn_CR(sock_num)));
}

/*******************************************************************************
 * Function Name  : macraw_send
 * Description    : This function used to send the data in MACRAW mode
 * Input          : Node
 * Output         : None
 * Return         : None
 *******************************************************************************/
uint16 macraw_send( const uint8 * buf, uint16 len )
{
    uint16 ret=0;
    uint16 ptr = 0;
    uint32 addrbsb = 0;

    if (len > getIINCHIP_TxMAX()) ret = getIINCHIP_TxMAX(); // check size not to exceed MAX size.
    else ret = len;

    ptr = IINCHIP_READ( Sn_TX_WR0(0) );
    ptr = ((ptr & 0x00ff) << 8) + IINCHIP_READ(Sn_TX_WR1(0));

    addrbsb = (uint32)(ptr<<8) + 0x10;
    wiz_write_buf(addrbsb, (uint8*)(buf), len);
    ptr += len;

    IINCHIP_WRITE( Sn_TX_WR0(0),(uint8)((ptr & 0xff00) >> 8));
    IINCHIP_WRITE( Sn_TX_WR1(0),(uint8)(ptr & 0x00ff));

    IINCHIP_WRITE(Sn_CR(0),Sn_CR_SEND);
    while( IINCHIP_READ(Sn_CR(0)) );
    while ( (IINCHIP_READ(Sn_IR(0)) & Sn_IR_SEND_OK) != Sn_IR_SEND_OK );
    IINCHIP_WRITE(Sn_IR(0), Sn_IR_SEND_OK);

    return ret;
}

/*******************************************************************************
 * Function Name  : macraw_recv
 * Description    : This function used to recvc the data in MACRAW mode
 * Input          : Node
 * Output         : None
 * Return         : None
 *******************************************************************************/
uint16 macraw_recv( uint8 * buf, uint16 len )
{
   
    uint16 data_len=0;
    uint8 dlen[2];
    uint16 ptr = 0;
    uint32 addrbsb = 0;

    if ( len > 0 )
    {

        ptr = IINCHIP_READ(Sn_RX_RD0(0));
        ptr = (ptr<<8) + IINCHIP_READ( Sn_RX_RD1(0) );
        addrbsb = (ptr<<8) + 0x10;
        wiz_read_buf(addrbsb, dlen, 2);
        data_len = (uint16)(dlen[0]<<8) + dlen[1];


        if(data_len > 1514)
        {
            printf("data_len over 1514\r\n");
            printf("\r\nptr: %X, data_len: %X", ptr, data_len);
            //while(1);
            /** recommand : close and open **/
            IINCHIP_WRITE( Sn_CR(0) ,Sn_CR_CLOSE);
            while( IINCHIP_READ(Sn_CR(0) ) );  
            Delay_ms(10);
            IINCHIP_WRITE(Sn_MR(0) ,Sn_MR_MACRAW);
            IINCHIP_WRITE( Sn_CR(0) ,Sn_CR_OPEN);
            while( IINCHIP_READ(Sn_CR(0)));

            return 0;
        }

        ptr += 2;
        addrbsb = (ptr<<8) + 0x10;
        wiz_read_buf(addrbsb, (uint8*)(buf), data_len);
        ptr += data_len;

        IINCHIP_WRITE(Sn_RX_RD0(0),(uint8)((ptr & 0xff00) >> 8));
        IINCHIP_WRITE(Sn_RX_RD1(0),(uint8)(ptr & 0x00ff));
        IINCHIP_WRITE(Sn_CR(0), Sn_CR_RECV);
        while( IINCHIP_READ(Sn_CR(0)) ) ;
    }

    return data_len;
}



